@@ -5,15 +5,21 @@ use URI;
 use JSON::Any;
 use AnyEvent::HTTP;
 use Net::Plurk::UserProfile;
+use Net::Plurk::OAuth;
 use DateTime;
 use Data::Dumper;
 
 use namespace::autoclean;
 
-has api_key => ( isa => 'Str', is => 'rw');
+has consumer_key => ( isa => 'Str', is => 'ro');
+has consumer_secret => ( isa => 'Str', is => 'ro');
 has cookie => ( isa => 'HashRef', is => 'rw', default => sub{{}});
-has api_user => ( isa => 'Net::Plurk::UserProfile', is => 'rw');
-has api_errormsg => ( isa => 'Maybe[Str]', is => 'rw');
+has oauth => (
+    isa => 'Net::Plurk::OAuth',
+    is => 'rw',
+    lazy_build => 1,
+    builder => '_init_oauth');
+has own_profile => ( isa => 'Net::Plurk::UserProfile', is => 'rw');
 has publicProfiles => ( isa => 'HashRef', is => 'rw', default => sub {{}});
 #has plurks => ( isa => 'ArrayRef[Net::Plurk::Plurk]', is => 'rw', default => sub {[]});
 #has plurk_users => ( isa => 'HashRef[Net::Plurk::User]', is => 'rw', default => sub {{}});
@@ -29,7 +35,7 @@ has unreadAll => ( isa => 'Int', is => 'ro' );
 has unreadMy => ( isa => 'Int', is => 'ro' );
 has unreadPrivate => ( isa => 'Int', is => 'ro' );
 has unreadResponded => ( isa => 'Int', is => 'ro' );
-has json_parser => (isa => 'JSON::Any', is => 'ro', default => sub {JSON::Any->new()});
+has raw_output => ( isa => 'Int', is => 'rw', default => 0);
 
 =head1 NAME
 
@@ -37,11 +43,11 @@ Net::Plurk - A perl interface to Plurk API
 
 =head1 VERSION
 
-Version 0.09
+Version 0.10
 
 =cut
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 =head1 SYNOPSIS
 
@@ -50,11 +56,14 @@ Quick summary of what the module does.
 Perhaps a little code snippet.
 
     use Net::Plurk;
-    my $api_key = $PLURKAPIKEY // "dKkIdUCoHo7vUDPjd3zE0bRvdm5a9sQi";
-    my $user = $PLURKUSER // 'nobody';
-    my $pass = $PLURKPASS // 'nopass';
-    my $p = Net::Plurk->new(api_key => $api_key);
-    my $profile = $p->login(user => $user, pass => $pass );
+    my $key = $CONSUMER_KEY;
+    my $secret = $CONSUMER_SECRET;
+    my $access_token = $ACCESS_TOKEN
+    my $access_secret = $ACCESS_TOKEN_SECRET 
+    my $p = Net::Plurk->new(consumer_key => $key, consumer_secret => $secret);
+    my $p->authorize(access_token => $access_token,
+	access_token_secret => $access_secret)
+    my $profile = $p->get_own_profile();
     $p->add_events(
         on_new_plurk => sub {
             my $plurk = shift;
@@ -66,7 +75,7 @@ Perhaps a little code snippet.
         },
         );
     $p->listen;
-    my $json = $p->api( path => '/api');
+    my $json = $p->callAPI( '/api');
     ...
 
 =head1 EXPORT
@@ -76,84 +85,68 @@ if you don't export anything, such as for a purely object-oriented module.
 
 =head1 FUNCTIONS
 
-=head2 api
+=head2 callAPI
 
 Everything from here
 
 =cut
 
-sub api {
+sub _init_oauth {
+    my ($self) = @_;
+    return Net::Plurk::OAuth->new(
+	consumer_key => $self->consumer_key,
+	consumer_secret => $self->consumer_secret
+    );
+}
+
+=head2 errorcode
+
+return errorcode
+
+=cut
+
+sub errorcode {
+    my ($self) = @_;
+    return $self->oauth->_errorcode;
+}
+
+=head2 errormsg
+
+return errormsg
+
+=cut
+
+sub errormsg {
+    my ($self) = @_;
+    return $self->oauth->_errormsg;
+}
+
+=head2 authorize
+
+authorize with access token/secret
+
+=cut
+
+sub authorize {
     my ($self, %args) = @_;
+    return $self->oauth->authorize(
+	access_token => $args{'token'} || $args{'access_token'},
+	access_token_secret => $args{'token_secret'} || $args{'access_token_secret'},
+    )
+};
+
+=head2 callAPI
+
+Everything from here
+
+=cut
+
+sub callAPI {
+    my ($self, $path, %args) = @_;
     my ($data, $header);
-    my $w = AE::cv;
-    my $u = URI->new("http://www.plurk.com/");
-    $u->path("/API".$args{path});
-    $u->scheme("https") if index($args{path}, "/Users") == 0;
-    delete $args{path};
-    $args{api_key} = $self->api_key unless $args{api_key};
-    $u->query_form(%args);
-    http_get ($u, 
-        cookie_jar => $self->cookie,
-        sub {
-            ($data, $header) = @_;
-            $self->api_errormsg(undef); # clear errormsg
-            $data = $self->json_parser->from_json($data);
-            $self->api_errormsg($data->{error_text}) if $header->{Status} eq '400';
-            $w->send;
-        }
-    );
-    $w->recv;
-    return wantarray ? ($data, $header) : $data;
-}
 
-=head2 is_logged_in
-
-Check if logged in, we use /API/Blocks/get to check
-the return JSON contains 'error_text' => 'Requires login'
-
-=cut
-
-sub is_logged_in {
-    my $self = shift;
-    return 0 unless $self->cookie;
-    my $json_data = $self->api(
-        path => '/Blocks/get',
-    );
-    return 0 if /error_text/ ~~ %{$json_data};
-    return 1;
-}
-
-=head2 login
-
-Get the request url and return Net::Plurk::User 
-
-=cut
-
-sub login {
-    my ($self, %args) = @_;
-    $args{no_data} //= 0;
-    my $json_data = $self->api(
-        path => '/Users/login',
-        username => $args{user},
-        password => $args{pass},
-#        no_data => $args{no_data} if $args{no_data},
-    );
-    $self->api_user(Net::Plurk::UserProfile->new($json_data));
-    return $self->api_user;
-}
-
-=head2 logout
-
-Just logout, should alway return 1
-
-=cut
-
-sub logout {
-    my ($self, %args) = @_;
-    my $json_data = $self->api(
-        path => '/Users/logout',
-    );
-    return $json_data->{'success_text'} eq 'ok'; 
+    $path = '/APP'.$path unless $path =~ m#/(?:APP|OAuth)#;
+    return $self->oauth->request($path, %args);
 }
 
 =head2 _get_unique_id
@@ -177,14 +170,31 @@ call /Profile/getPublicProfile
 
 sub get_public_profile {
     my ($self, $user) = @_;
-    my $json_data = $self->api(
-        path => '/Profile/getPublicProfile',
+    my $json_data = $self->callAPI(
+        '/Profile/getPublicProfile',
         user_id => $user,
     );
+    return $json_data if $self->raw_output;
     $self->publicProfiles->{ $user } = Net::Plurk::PublicUserProfile->new($json_data);
     return $self->publicProfiles->{ $user };
 }
 
+=head2 get_own_profile
+
+call /Profile/getOwnProfile
+
+=cut
+
+sub get_own_profile {
+    my ($self, $user) = @_;
+    my $json_data = $self->callAPI(
+        '/Profile/getOwnProfile',
+    );
+    return $json_data if $self->raw_output;
+    $self->own_profile(Net::Plurk::UserProfile->new($json_data));
+    return $self->own_profile;
+}
+
 =head2 get_new_plurks
 
 call /Polling/getPlurks
@@ -198,11 +208,12 @@ sub get_new_plurks {
     $args{offset} //= $self->lastPollingTime;
     $args{limit} //= 50; # default is 50
     $args{offset} = $args{offset}->strftime("%Y-%m-%dT%H:%M:%S");
-    my $json_data = $self->api(
-        path => '/Polling/getPlurks',
+    my $json_data = $self->callAPI(
+        '/Polling/getPlurks',
         offset => $args{offset},
         limit => $args{limit},
     );
+    return $json_data if $self->raw_output;
     $self->plurk_users($json_data->{plurk_users});
     $self->plurks($json_data->{plurks});
     $self->lastPollingTime(DateTime->now);
@@ -211,20 +222,21 @@ sub get_new_plurks {
 
 =head2 karma
 
-    return logined user's karma, or specify user => 'who'
+    return user's karma, or specify user => 'who'
 
 =cut
 
 sub karma {
     my ($self, %args) = @_;
     return $self->get_public_profile($args{user})->user_info->karma if $args{user};
-    return 0 unless $self->is_logged_in();
-    return $self->api_user->user_info->karma;
+    return 0;
+    # TODO: if authorized, return own karma, otherwise fail?
+    #return $self->own_profile->user_info->karma;
 }
 
 =head2 follow
 
-    return 1 if followed someone, 0 otherwise (see api_errormsg)
+    return 1 if followed someone, 0 otherwise (see errormsg)
 
 =cut
 
@@ -234,18 +246,19 @@ sub follow {
     # if input user nick_name instead of unique id
     $user_id = $self->_get_unique_id($user_id) if $user_id !~ m/^\d+$/;
 
-    my $json_data = $self->api(
-        path => '/FriendsFans/setFollowing',
+    my $json_data = $self->callAPI(
+        '/FriendsFans/setFollowing',
         user_id => $user_id,
         follow => 'true',
     );
-    return 0 if $self->api_errormsg;
+    return $json_data if $self->raw_output;
+    return 0 if $self->errormsg;
     return 1;
 }
 
 =head2 unfollow
 
-    return 1 if unfollowed someone, 0 otherwise (see api_errormsg)
+    return 1 if unfollowed someone, 0 otherwise (see errormsg)
 
 =cut
 
@@ -255,12 +268,13 @@ sub unfollow {
     # if input user nick_name instead of unique id
     $user_id = $self->_get_unique_id($user_id) if $user_id !~ m/^\d+$/;
 
-    my $json_data = $self->api(
-        path => '/FriendsFans/setFollowing',
+    my $json_data = $self->callAPI(
+        '/FriendsFans/setFollowing',
         user_id => $user_id,
         follow => 'false',
     );
-    return 0 if $self->api_errormsg;
+    return $json_data if $self->raw_output;
+    return 0 if $self->errormsg;
     return 1;
 }
 
@@ -274,13 +288,14 @@ sub unfollow {
 sub add_plurk {
     my ($self, $content, $qualifier, %opt) = @_;
     $qualifier //= 'says';
-    my $json_data = $self->api(
-        path => '/Timeline/plurkAdd',
+    my $json_data = $self->callAPI(
+        '/Timeline/plurkAdd',
         qualifier => $qualifier,
         content => $content,
         %opt,
     );
-    return Net::Plurk::Plurk->new($json_data) if !$self->api_errormsg;
+    return $json_data if $self->raw_output;
+    return Net::Plurk::Plurk->new($json_data) if !$self->errormsg;
     return ;
 }
 
@@ -332,7 +347,7 @@ L<http://search.cpan.org/dist/Net-Plurk/>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009~2010 Cheng-Lung Sung, all rights reserved.
+Copyright 2009~2011 Cheng-Lung Sung, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
